package org.wsi.test.monitor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.wsi.WSIException;
import org.wsi.test.log.Log;
import org.wsi.test.monitor.config.MonitorConfig;

/** This class is a simple wrapper (also in the <code>org.wsi.test.monitor</code>
 * package) that exposes some of the protected- and package- methods of 
 * @link{org.wsi.test.monitor.Monitor} in order to try and make it more usable 
 * from third-party classes that may want to use WS-I testing tools functionality 
 * in a more flexible manner than the solution provided.
 * 
 * For example, one may want to provide a friendlier GUI in order to configure 
 * and/or run the testion tools, as well as distribute it as part of a more 
 * holistic validation solution.
 * 
 * To this end, this class also provides JavaBeans-style property change event
 * notification:
 * 
 *   RUNNING_PROPERTY: Status of monitor (running or not)
 *   MESSAGE_PROPERTY: Messages output by the monitor (methinks)
 * 
 * NOTE: Because of the lack of documentation on the part of WS-I, most of the 
 * functionality of this class is ether guessed or deduced through 
 * experimentation.
 * 
 * @author STCD (www.solms.co.za)
 * @version 0.1 (Jun 13, 2004)
 */
public class MonitorWrapper extends Monitor implements PropertyChangeListener
{
    /** Constructs and starts a monitor, using the specified configuration
     * file. This is simply a convenience to having to produce your own String
     * array of arguments.
     * @param configFile Path to the configuration file (relative to what?)
     * @throws WSIException If something goes wrong.
     */
    public MonitorWrapper(String configFile) throws WSIException
    {
        this(makeConfigArray(configFile));
    }
    
    
    /** Constructs a monitor with the given arguments, as per
     * command-line arguments of @link{org.wsi.test.monitor.Monitor}
     * @param args Configuration, usually {"-config", "config_file_path"}
     * 
     */
    public MonitorWrapper(String[] args) throws WSIException
    {
    		// Call superclass (which starts the monitor)
        super(args);
        
        // Init Property change support
        pcSupp = new PropertyChangeSupport(this);
        addPropertyChangeListener(this);
    }    
 
    
    /** Indicates (with reasonable accuracy) whether this monitor is still running or not
     * ( e.g. exited because of manual shutdown or an error)
     * @return boolean, true if monitor is still accepting messages, or false otherwise.
     */
    public boolean isRunning()
    {
    		return running;
    }
    
    
    /** Changes the state of whether the monitor is running or not
     */
    protected void setRunning(boolean r)
    {
	    	boolean oldVal = running;
	    	running = r;
	    	try
	    	{
	    	    pcSupp.firePropertyChange(RUNNING_PROPERTY,oldVal,running);
	    	}
	    	catch (RuntimeException ignored){}
    }
    
    
    /** Gets the runtime status. This is a string as defined by the WS-I monitor,
     * @return String, one of "start01","start02", "stopping02", "stopping01", "stopped01".
     */
    public String getStatus()
    {
        return this.status;
    }
    
    
    /** Sets the runtime status. This is based on output messages received from
     * the WS-I Monitr, and may not be 100% accurate;
     */
    public void setStatus(String status)
    {
        if (status == null)
            return;
        
        // Set value and keep old value
        String oS = this.status;
        this.status = status;
        
        // Set running property
        if (STATUS_STARTED.equals(status))
        {
            setRunning(true);
        }
        else
        if (STATUS_STOPPED.equals(status))
        {
            setRunning(false);
        }
        
        try
        {
            // Fire change
            pcSupp.firePropertyChange(STATUS_PROPERTY,oS,status);
        }
        catch (RuntimeException ignored){}
    }
    
    
    /** Called when the monitor exits.
     * @see org.wsi.test.monitor.Monitor#exitMonitor()
     */
    void exitMonitor()
    {
        super.exitMonitor();
        setRunning(false);
    }
    
    
    /** Called when the monitor exits.
     * @see org.wsi.test.monitor.Monitor#exitMonitor(java.lang.Exception)
     */
    void exitMonitor(Exception arg0)
    {
        super.exitMonitor(arg0);
        setRunning(false);
    }
    
    
    /** Returns the enxt available conversation ID (caution, use with care).
     * Calling this unnecessarily may break tools which expect a linear progression
     * of IDs in the log file.
     * @see org.wsi.test.monitor.Monitor#getNextConversationId()
     */
    public synchronized int getNextConversationId()
    {
        return super.getNextConversationId();
    }

    
    /** Causes the monitor to stop processing, close any log file(s), and exit gracefully
     * (bar a few messages written to <code>System.out</code> :-).
     */
    public void stopMonitor()
    {
        super.stopMonitor();
    }
    
    
    /** Returns a reference to the monitor's log.
     * @see org.wsi.test.monitor.Monitor#getLog()
     */
    public Log getLog()
    {
        return super.getLog();
    }
    
    
    /** Returns the current message entry queue (whatever that is, presumably
     * it's the messages which have not yet been written to file)
     * @see org.wsi.test.monitor.Monitor#getMessageEntryQueue()
     */
    public MessageEntryQueue getMessageEntryQueue()
    {
        return super.getMessageEntryQueue();
    }


	/** Returns the monitor's configuration.
     * @see org.wsi.test.monitor.Monitor#getMonitorConfig()
     */
    public MonitorConfig getMonitorConfig()
    {
        return super.getMonitorConfig();
    }
    
    
    /** Callled to presumably write a message to screen.
     * @see org.wsi.test.monitor.Monitor#printMessage(java.lang.String, java.lang.String)
     */
    public void printMessage(String arg0, String arg1)
    {
        super.printMessage(arg0, arg1);
    }
    
    
    /** Callled to presumably write a message to screen.
     * @see org.wsi.test.monitor.Monitor#printMessage(java.lang.String, java.lang.String, java.lang.String)
     */
    public void printMessage(String arg0, String arg1, String arg2)
    {
        // Let superclass do what it's supposed to
        super.printMessage(arg0, arg1, arg2);
        
        // First argument is the status, set it
        setStatus(arg0);
        
        // Send message (third argument)
        sendMessage(arg2);
    }   
    
    
    /** Sends a message event (e.g. print to console)
     * unless it's the stupid "Type 'exit' to stop..." one.
     * @param arg2
     */
    private void sendMessage(String msg)
    {
        // Check for ignored message
        if (msg == null || msg.indexOf("to stop the Monitor.") != -1)
            return;
        
        try
        {
            // Fire an event for the message
            pcSupp.firePropertyChange(MESSAGE_PROPERTY, null, msg);
        }
        catch (RuntimeException ignored){}
    }


    /** Presumably this message converts an XML document to a more "console-oriented"
     * String message (line by line) but this may be clarified by further testeing.
     * @see org.wsi.test.monitor.Monitor#toXMLString(java.lang.String)
     */
    public String toXMLString(String arg0)
    {
        return super.toXMLString(arg0);
    }
    
    
    /** Allows an object to register itself to receive property change events
     */
    public void addPropertyChangeListener(PropertyChangeListener l)
    {
	    	if (l != null)
	    	{
	    		pcSupp.addPropertyChangeListener(l);
	    	}
    }


	/** Unregisters a consumer of property change events.
	 */    
    public void removePropertyChangeListener(PropertyChangeListener l)
    {
    		pcSupp.removePropertyChangeListener(l);
    }
    
    
    /* (non-Javadoc)
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt)
    {
        System.out.println("[CHANGE]: " + evt.getPropertyName() + ": " + evt.getNewValue());
        
    }
    
    /** Converts a singe string config file name to an array of parameters
     * for the constructor. 
     * @param configFile
     * @return
     */
    private static String[] makeConfigArray(String configFile)
    {
        String[] args = {"-config", configFile};
        return args;
    }
    
    
    
    
    /** Property: Is the monitor currently "running" or not?
     */
    public static final String RUNNING_PROPERTY = "running";
    
    /** Property: What is the monitor's status (e.g starting up, stopped..)
     */
    public static final String STATUS_PROPERTY = "status";
    
    /** Property: Used to convey messages as they are printed.
     * Though not strictly a property, this is the only reasonable method
     * to intercept WS-I test tools messages. 
     */
    public static final String MESSAGE_PROPERTY = "message";
    
    // Runtime Status
    public static final String STATUS_STARTING = "start01";
    public static final String STATUS_STARTED = "start02";
    public static final String STATUS_STOPPING = "stopping01";
    public static final String STATUS_STOPPED = "stopped01";
    private String status = STATUS_STOPPED;
    
    // Indicates whether the monitor is still running (with reasonable accuracy)
    private boolean running = false;
    
    // Property change support
    private PropertyChangeSupport pcSupp;
}
