package solms.xmlvend;

import java.io.File;
import java.io.IOException;
import java.util.*;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import org.wsi.log.Log;
import solms.xmlvend.messagevalidator.ReportGenerator;
import solms.xmlvend.messagevalidator.ReportPersistenceManager;
import solms.xmlvend.messagevalidator.impl.FileReportPersistenceManager;
import solms.xmlvend.messagevalidator.impl.LogUnmarshalValidationHandler;
import solms.xmlvend.messagevalidator.impl.ReportGeneratorImpl;
import solms.xmlvend.messagevalidator.impl.ReportKeyGeneratorImpl;
import solms.xmlvend.messagevalidator.impl.TestReportTransformer;
import solms.xmlvend.testreport.Testreport;
import za.co.solms.config.ConfigException;
import za.co.solms.config.ConfigHelper;
import za.co.solms.config.Configurable;

/**
 * Main application to produce reports, based on there being a pre-existing
 * WS-I Monitor Log File that it can process. This is currently a very precedural, "main"
 * style application for testing purposes - it should be broken up into smaller components
 * used in a more object-oriented manner.
 *
 * @author dawidl
 * @version 0.1 (Jul 2, 2004)
 */
public class ReportRunner
{

    public static void main(String[] args)
    {
        // Check arguments
        if (args.length < 2)
        {
            System.out.println("ReportRunner: Required arguments: <configFile> <logFile>");
            System.exit(1);
        }
        
        // Read configuration
        Properties config = null;
        try
        {
            config = ConfigHelper.getFromFile(args[0]);
            if (config == null) throw new ConfigException("No configuration info at " + args[0]);
        } catch (Exception e)
        {
            System.err.println("Could not read configuration: " + e);
            System.exit(1);
        }
        
        // Check whether logfile exists
        File logFile = new File(args[1]);
        if (!logFile.exists() || !logFile.canRead())
        {
            System.err.println("Cannot read: " + logFile);
            System.exit(1);
        }
        
        // If output dir doesn't exist, create it
        File outputDir = new File(config.getProperty("solms.xmlvend.reportoutputdir"));
        if (outputDir.exists())
        {
            if (!outputDir.isDirectory())
            {
                System.err.println("Not a directory: " + outputDir);
                System.exit(1);
            }            
        }
        else
        {
            // Create it
            try
            {
                outputDir.mkdir();
                System.out.println("Created output directory: " + outputDir);
            } catch (Exception e)
            {
                System.err.println("Could not create directory: " + outputDir);
                System.exit(1);
            }
        }
        
        
        // Read the log as WS-I XML (and parse to objects)
        Log log = null;
        try
        {
            System.out.println("Parsing log file...");
            JAXBContext context = JAXBContext.newInstance("org.wsi.log");
            Unmarshaller u = context.createUnmarshaller();
            u.setEventHandler(new LogUnmarshalValidationHandler());
            u.setValidating(false);
            log = (Log)u.unmarshal(logFile);
        }
        catch (JAXBException e)
        {
            System.err.println("Error(s) in log file: " + e);
            System.err.println("Aborting.");
            System.exit(1);
        }
        
        // Generate Report(s)
        Collection reports = null;
        try
        {
            System.out.println("Generating Report(s)...");
            // Create and Configure the report generator
	        ReportGenerator reportGenerator = new ReportGeneratorImpl();
	        ((Configurable)reportGenerator).configure(config);
	        reports = reportGenerator.generateReports(log);
        } catch (Exception e)
        {
            System.err.println("Report Generation failed: " + e);
            System.exit(1);
        }
        
        try
        {
            // Write to their respective files
            System.out.println("Persisting Report(s)...");
            Iterator reps = reports.iterator();
            ReportPersistenceManager p = new FileReportPersistenceManager(outputDir);
            
            // Apply configuration if the implementing persistence class supports it
            try
            {
                ((Configurable)p).configure(config);
            }
            catch (ClassCastException ignored){}
            catch (ConfigException e)
            {
                System.err.println("Error configuring ReportPersistenceManager: " + e + " (aborting)");
                System.exit(1);
            }
            // Cycle through and write reports one by one
            while (reps.hasNext())
            {
                Testreport r = (Testreport)reps.next();
                try
                {
                    p.persist(r);
                } catch (Exception e)
                {
                    System.err.println("Failed writing report: " + new ReportKeyGeneratorImpl().generateKey(r));
                    System.err.println(e);
                }
            }
        }
        catch (IOException ioe)
        {
            System.err.println("Report Persistence failed: " + ioe);
            System.exit(1);
        }
        
        
        // Transform if there is an XSLT Style Sheet specified
        if (config.getProperty("solms.xmlvend.reportxslt") != null)
        {
            System.out.println("Transforming Report(s)...");
            try
            {
                TestReportTransformer transformer = new TestReportTransformer();
                transformer.configure(config);
                transformer.transformAll(outputDir);
	         }
	         catch (Exception e)
	         {
	             System.err.println("Report Transformation failed: " + e);
	             System.exit(1);
	         }
        }
        
        // Made it this far? Have a beer...
        System.out.println("Done.\n");
    }
}
