package solms.xmlvend.messagevalidator.impl;

import java.util.*;
import org.wsi.log.HttpMessageEntry;
import solms.xmlvend.messagevalidator.*;
import solms.xmlvend.messagevalidator.util.MessageEntryDateComparator;
import solms.xmlvend.testreport.Testreport;
import za.co.solms.config.ConfigException;
import za.co.solms.config.Configurable;

/**
 * Generates reports by splitting them up according to ClientID + Server IP Address.
 * This generator uses a simple yet pluggable mechanism which allow for multiple validators
 * to be plugged in per message.
 * 
 * This class is Configurable, and the set of validators to use is specified by the 
 * "solms.xmlvend.reportvalidators" configuration property.
 *
 * @author dawidl
 * @version 0.1 (Jul 1, 2004)
 */
public class ReportGeneratorImpl implements ReportGenerator, Configurable
{
    // Identifier
    public static final String ID = "ReportGeneratorImpl";
    // Report splitter
    private ReportSplitter reportSplitter;
    // Configuration
    private Properties configuration;
    // Report Validators
    private List reportValidators;
    
    
    /** Default Constructor */ 
    public ReportGeneratorImpl()
    {
        // Create report splitter
        this.reportSplitter = new ReportSplitterImpl();
    }

    /* Generates a collection of reports based on a WS-I Log.
     * @see solms.xmlvend.messagevalidator.ReportGenerator#generateReports(org.wsi.log.LogType)
     * @throws IllegalArgumentException If wsiLog is null.
     */
    public Collection generateReports(org.wsi.log.LogType wsiLog) throws Exception
    {
        if (wsiLog == null) throw new IllegalArgumentException("Invalid log (null) provided.");
        
        // Create TreeMap of messages (order and associate requests and reponses)
        Collection messages = wsiLog.getMessageEntry();
        SortedMap sortedMessages = associateMessages(messages);
              
        // Create TreeMap of reports, given a TreeMap of messages.
        // Keys are Strings, Values are reports
        SortedMap rawReports = generateRawReports(sortedMessages);
        
        // Ask each validator to validate each report
        Collection reports = rawReports.values();
        validateReports(reports);
                
        return reports;
    }
    
    
    /** Generates a SortedMap of MessageEntries (keys and values) given a collection of
     * MessageEntries. Please see the documentation for the Algorithm explanation.
     */
    public SortedMap associateMessages(Collection c)
    {        
        // Create messages TreeMap, order by date
        MessageEntryDateComparator comp = new MessageEntryDateComparator();
        TreeMap messageMap = new TreeMap(comp);
        
        // If invalid argument, return empty map
        if (c == null) return messageMap;
        
        // Cycle through all messages
        Iterator i = c.iterator();        
        while (i.hasNext())
        {
            Object next = i.next();
            try
            {                
                // Cast to HTTP Message
                HttpMessageEntry msg = (HttpMessageEntry)next;
                if ("request".equals(msg.getType()))
                {
                    // Add as key
                    messageMap.put(msg,null);
                }
                else if ("response".equals(msg.getType()))
                {
                    // Find matching key, and add as value
                    HttpMessageEntry matchingRequest = findMatchingRequest(msg, messageMap);
                    messageMap.put(matchingRequest, msg);
                }
            }
            catch (ClassCastException ignored)
            {
                System.err.println(ID+": ** Could not cast message as HttpMessageEntry: "+next.getClass().getName());
            }
        }        
        return messageMap; 
    }
    
    
    /** Generates a SortedMap of TestReport instances. These have yet to be processed
     * by validators.
     * @param messages SortedMap of MessageEntries (keys and values)
     */
    public SortedMap generateRawReports(SortedMap messages) throws Exception
    {
        return reportSplitter.produceRawReports(messages);
    }
    
    
    /** Returns the mathcing request for a given response message.
     */
    public HttpMessageEntry findMatchingRequest(HttpMessageEntry response, SortedMap map)
    {
        // TODO Needs decent error handling
        
        // Add all matching irems in the map's keyset to a list, which match the ConversationID and
        // which don't have a matching response already.
        java.util.List candidates = new ArrayList();
        Set requests = map.keySet();
        Iterator i = requests.iterator();
        while (i.hasNext())
        {
            HttpMessageEntry candidate = (HttpMessageEntry)i.next();
            if (map.get(candidate) == null)
            {
                if(candidate.getConversationID().equals(response.getConversationID()))
                {
                    candidates.add(candidate);
                }
            }
        }
        
        // Take the last candidate in the list (latest date/time) or return null if none
        return candidates.size() < 1 ? null : (HttpMessageEntry)candidates.get(candidates.size() - 1);
    }
    
    
    /** Returns a List of ReportValidator instances which will be applied to
     * validate the report. Only when this method gets called will the classes
     * be loaded and instantiated. If the validators implement the Configurable interface,
     * the current configuration will be applied.
     * TODO Should this fail-fast if one validator cannot load, or continue fault-tolerantly?
     * It is currently set to fail-fast (if something goes wrong, an exception is thrown)
     */
    public java.util.List getReportValidators() throws Exception
    {
        // Are we configured?
        if (configuration == null)
            throw new ConfigException("No configuration has been applied.");
        
        // Comma-separate list of validators
        String classes = configuration.getProperty("solms.xmlvend.reportvalidators");
        List validators = new ArrayList();
        if (classes == null || classes.trim().length() < 1)
            return validators;
        
        // Cycle through classnames
        StringTokenizer t = new StringTokenizer(classes,",");
        while (t.hasMoreTokens())
        {
            String className = t.nextToken().trim();
            // Only try to instantiate non-empty classname (e.g. ignore two consecutive commas)
            if (className != null && className.length() > 0)
            {
	            Object o = Class.forName(className).newInstance();
	            try
	            {
	                // Try to cast and configure
	                ((Configurable)o).configure(this.configuration);
	            }
	            catch (ClassCastException ignored){}
	            // Cast to ReportValidator, and add to list of validators
	            validators.add((ReportValidator)o);
            }
        }        
        
        return validators;
    }
    
    
    /** Upon configuration, all report validators will also be
     * created.
     * @see za.co.solms.config.Configurable#configure(java.util.Properties)
     */
    public void configure(Properties configuration) throws ConfigException
    {
        // Check configuration
        if (configuration == null)
            throw new ConfigException("Invalud configuration (null)");
        
        // Set (copy for safety)
        this.configuration = (Properties)configuration.clone();
        
        // Create report validators
        try
        {
            this.reportValidators = getReportValidators();
        } catch (Exception e)
        {
            throw new ConfigException("Could not create report validators as per configuration: " + e);
        }
    }
    
    
    /**
     * Will apply validation to the given collection of reports, using the internal
     * set of validators as per the current configuration. Does not return anything,
     * as the validation will alter the TestReport instances themselves, i.e. by attaching
     * Error elements to the ServiceRequests.
     */
    public void validateReports(Collection reports) throws ReportValidationException
    {
        // Check if we have validators (zero is OK, but if it's null, we've not been configured)
        if (this.reportValidators == null)
            throw new ReportValidationException("This ReportGenerator has not yet been configured (No Validators).");
        
        // Check if we have reports
        if (reports == null)
            return;
        
        // Cycle through reports, and then apply each validator in sequence
        Iterator reps = reports.iterator();
        while (reps.hasNext())
        {
            // The current report
            Testreport report = (Testreport)reps.next();
            Iterator validators = reportValidators.iterator();
            while (validators.hasNext())
            {
                ReportValidator validator = (ReportValidator)validators.next();
                try
                {
                    // Validate
                    validator.validate(report);
                }
                catch (ReportValidationException e)
                {
                    System.err.println("**ERROR Validating: " + validator.getClass().getName() + ": " + e);
                    // TODO Need to figure out what to do with these errors, as we don't want to kill the whole process
                }
            }
        }        
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getConfiguration()
     */
    public Properties getConfiguration()
    {
        return configuration;
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getRequiredKeys()
     */
    public String[] getRequiredKeys()
    {
        String[] keys = {"solms.xmlvend.reportvalidators"};
        return keys;
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getOptionalKeys()
     */
    public String[] getOptionalKeys()
    {
        return null;
    }
}
