package solms.xmlvend.messagevalidator.impl;

import java.io.File;
import java.io.IOException;
import java.util.Properties;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import solms.xmlvend.messagevalidator.util.XVReportFileFilter;
import za.co.solms.config.ConfigException;
import za.co.solms.config.Configurable;
import za.co.solms.io.files.Finder;

/**
 * Transforms test reports using XSLT. The style sheet URI is set by the property
 * "solms.xmlvend.reportxslt". 
 *
 * @author dawidl
 * @version 0.1 (Jul 5, 2004)
 */
public class TestReportTransformer implements Configurable
{
    private Properties configuration;
    private Transformer transformer;
    
    /** Transforms all files in the specified directory (recursively).
     *
     */
    public void transformAll(File directory) throws IOException, TransformerException
    {
        // Check dir
       if (directory == null || directory.isFile() || !directory.exists())
           throw new IOException("Invalid directory to transform XSLT: " + directory);
       
       // Use Solms finder component to get all files recursively
       Finder finder = new Finder();
       java.util.Collection files = finder.listFilesRecursively(directory, new XVReportFileFilter());
       
       // Cycle through them, and transform them accordingly
       java.util.Iterator filesIterator = files.iterator();
       while (filesIterator.hasNext())
       {
           transform((File)filesIterator.next());
       }
    }
    
    
    /** Transforms a specific file, using the configured
     * XSLT and file extension.
     * 
     */
    public void transform(File file) throws TransformerException
    {
        // Determine file name of output
        File transformedFile = getTransformedName(file);
        
        // Set up source and output streams for XSLT Parsing
        StreamSource source = new StreamSource(file);
        StreamResult result = new StreamResult(transformedFile);
        
        transformer.transform(source, result);
        System.out.println(" Wrote report: " + transformedFile.toString());
    }
    

    /** Determines the new file being transformed from the given file.
     * @param file
     * @return
     */
    private File getTransformedName(File file)
    {
        String ext = configuration.getProperty("solms.xmlvend.reportxsltextension");
        if (ext == null) ext = "-transformed.xml";
        
        String fName = file.getName().trim();
        int lastDot = fName.lastIndexOf(".");
        if (lastDot != -1)
        {
            return new File(file.getParentFile(), fName.substring(0,lastDot) + ext);
        }
        else
        {   
            return new File(file.getParentFile(), fName + ext);
        }
    }


    /** Sets up the XSLT Transformer etc.
     */
    public void configure(Properties arg0) throws ConfigException
    {
        this.configuration = (Properties)arg0.clone();
        
        // Setup XSLT and DOM transformation
        StreamSource styleSource = new StreamSource(new File(configuration.getProperty("solms.xmlvend.reportxslt")));
        TransformerFactory tfactory = TransformerFactory.newInstance();
        try
        {
            this.transformer = tfactory.newTransformer(styleSource);
        }
        catch (TransformerConfigurationException e)
        {
            throw new ConfigException("Could not configure XSLT Transformer: " + e);
        }
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getConfiguration()
     */
    public Properties getConfiguration()
    {
        return (Properties)configuration.clone();
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getRequiredKeys()
     */
    public String[] getRequiredKeys()
    {
        // TODO Auto-generated method stub
        return null;
    }

    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getOptionalKeys()
     */
    public String[] getOptionalKeys()
    {
        // TODO Auto-generated method stub
        return null;
    }
}
