package solms.xmlvend.messagevalidator.validators;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.regex.Pattern;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import solms.xmlvend.messagevalidator.ReportValidationException;
import solms.xmlvend.messagevalidator.ReportValidator;
import solms.xmlvend.testreport.ObjectFactory;
import solms.xmlvend.testreport.RawMessage;
import solms.xmlvend.testreport.RequestError;
import solms.xmlvend.testreport.Response;
import solms.xmlvend.testreport.ResponseError;
import solms.xmlvend.testreport.ServiceRequest;
import solms.xmlvend.testreport.Testreport;
import za.co.solms.config.ConfigException;
import za.co.solms.config.ConfigHelper;
import za.co.solms.config.Configurable;

/**
 * This validator validates a SOAP message using straight-forward schema validation. It requires
 * a directory (to source schema files from), which it will load and use to perform namespace-aware
 * schema validation. This includes, in addition to soap messages / content, the SOAP schemas as well,
 * in order to validate the entire message.
 *
 * @author dawidl
 * @version 0.2 (Jul 4, 2004 - Updated Jan 30, 2006)
 */
public class SOAPBodySchemaValidator implements ReportValidator, Configurable
{
    // Configuration information
    private Properties configuration;
    // XML Parsing
    private DefaultHandler handler;
    private SAXParser parser;
    private String messageExclusionPattern;
    private Pattern mxPattern;
    static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema"; 
    static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
    private static final String ERROR_PREFIX = "XML Schema validation: ";
    private static FileFilter schemaFilter = new SchemaFileFilter();

    
    /* (non-Javadoc)
     * @see solms.xmlvend.messagevalidator.ReportValidator#getDescription()
     */
    public String getDescription()
    {
        return "Schema Validator";
    }

    
    /* (non-Javadoc)
     * @see solms.xmlvend.messagevalidator.ReportValidator#validate(solms.xmlvend.testreport.Testreport)
     */
    public void validate(Testreport report) throws ReportValidationException
    {
        Iterator messages = report.getServiceRequest().iterator();
        while (messages.hasNext())
        {
            ServiceRequest req = (ServiceRequest)messages.next();
            // Validate
            try
            {
                validate(req);
            } catch (Exception e)
            {
                System.err.println("**ERROR: During Schema Validation: " + e + " (ignoring)");
            }
        }
    }
    
    
    /** Validates the particular service request message, and it's response (if any)
     */
    public void validate(ServiceRequest request) throws Exception
    {
        if (request == null) return;
        
        // REQUEST
        String reqMessage = request.getRawRequest().getMessage();
        ByteArrayInputStream reqIn = new ByteArrayInputStream(reqMessage.getBytes());
        try
        {
            parser.parse(reqIn, handler);
        }
        catch (MultiSAXException m)
        {
            // Contains multiple errors - add them individually
            for (Iterator i = m.getExceptions().iterator(); i.hasNext();)
            {
                try
                {
                    SAXException e = (SAXException)i.next();
                    String desc = e.getMessage();
                    
                    // Add error element
                    ObjectFactory f = new ObjectFactory();
                    RequestError re = f.createRequestError();
                    re.setDescription(ERROR_PREFIX + desc);
                    request.getError().add(re);
                }
                catch (Exception ignored){}
            }
            
        }
        catch (SAXException e)
        {
            // Create error element, and add it
            ObjectFactory f = new ObjectFactory();
            RequestError re = f.createRequestError();
            re.setDescription(ERROR_PREFIX + e.getMessage());
            request.getError().add(re);
        }
        
        // RESPONSE
        Response response = request.getResponse();
        if (response != null)
        {
            RawMessage rsrm = response.getRawResponse();
            if (rsrm != null)
            {
                String resMessage = rsrm.getMessage();
                ByteArrayInputStream resIn = new ByteArrayInputStream(resMessage.getBytes());
                try
                {
                    parser.parse(resIn, handler);
                }
                catch (MultiSAXException m)
                {
                    // Contains multiple errors - add them individually
                    for (Iterator i = m.getExceptions().iterator(); i.hasNext();)
                    {
                        try
                        {
                            SAXException e = (SAXException)i.next();
                            String desc = e.getMessage();
                            
                            // Add error element
                            ObjectFactory f = new ObjectFactory();
                            ResponseError re = f.createResponseError();
                            re.setDescription(ERROR_PREFIX + desc);
                            response.getError().add(re);
                        }
                        catch (Exception ignored){}
                    }
                    
                }                
                catch (SAXException e)
                {
                    // Create error element, and add it
                    ObjectFactory f = new ObjectFactory();
                    ResponseError re = f.createResponseError();
                    re.setDescription(ERROR_PREFIX + e.getMessage());
                    response.getError().add(re);
                }
            }
        }
    }


    /** Upon configuration, the relevant schemas will be loaded and made available to the
     * JAXP validating parser.
     */
    public void configure(Properties config) throws ConfigException
    {
	    	try
	    	{
	        // Check configuration
	        if (config == null || !ConfigHelper.containsAll(config, getRequiredKeys()))
	        {
	            throw new ConfigException("Invalid Configuration, required values not present.");
	        }
	        this.configuration = (Properties)config.clone();
	        
	        // Determine schemas to load (read directory)
	        String sDir = configuration.getProperty("xmlvend.SOAPBodySchemaValidator.schemaDir");
	        String[] schemas = getSchemaURIsForValidation( new File(sDir) );
	        
	        // Setup XML Parsing
	        // TODO Pre-load schemas as byte arrays for optimum performance
	        // (Currently only the String URIs are loaded, leaving the caching to the parser)
	        try
	        {
	            handler = new SOAPParserHandler();
	            SAXParserFactory factory = SAXParserFactory.newInstance();
	            factory.setValidating(true);
	            factory.setNamespaceAware(true);
	            parser = factory.newSAXParser();
	            // Set to use W3C Schemas
	            parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
	            parser.setProperty(JAXP_SCHEMA_SOURCE, schemas); 
	        }
	        catch (Exception e)
	        {
	            throw new ConfigException("Error settup up XML Parser: " + e);
	        }
	    	}
	    	catch (IOException e)
	    	{
	    		throw new ConfigException( e.toString() );
	    	}
    }


    /** Produces a JAXB-ready list of schemas, by examining the given directory.
     * Does not recurse.
     * @param sourceDir
     * @return A string array.
     */
    public String[] getSchemaURIsForValidation( File sourceDir ) throws IOException
    {
    		// Java 5 Dependency
    		List<String> fileURIs = new ArrayList<String>();
    		if (sourceDir.isDirectory() && sourceDir.canRead())
    		{
    			File[] list = sourceDir.listFiles( schemaFilter );
    			for (File f : list)
    			{
    				if (f.isFile())
    				{
    					fileURIs.add( f.getAbsolutePath() );
    				}
    			}
    		}
    		System.out.println(" Validation: Using " + fileURIs.size() + " schema(s) from " + sourceDir.toString());
    		return fileURIs.toArray( new String[0] );
    }
    
    
    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getConfiguration()
     */
    public Properties getConfiguration()
    {
        return (Properties)configuration.clone();
    }


    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getRequiredKeys()
     */
    public String[] getRequiredKeys()
    {
        String[] keys = {"xmlvend.SOAPBodySchemaValidator.schemaDir"};
        return keys;
    }


    /* (non-Javadoc)
     * @see za.co.solms.config.Configurable#getOptionalKeys()
     */
    public String[] getOptionalKeys()
    {
        String[] keys = {	"xmlvend.SOAPBodySchemaValidator.ignorePattern"};
        return keys;
    }
    
    /** Filters files, only accepts XSD files and directories. Currently only does
     * simple filename-based checking.
     * 
     * @author dawidl
     * @version 0.1 (Jan 30, 2006)
     */
    public static class SchemaFileFilter implements FileFilter
    {
    		/** Accepts the given file if it's a directory, or name ends with ".xsd" */
	    	public boolean accept(File f)
	    	{
	    		return f.isDirectory() || f.getName().toLowerCase().endsWith(".xsd");
	    	}
    }

}