package solms.xmlvend.messagevalidator.impl;

import java.util.Iterator;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.wsi.log.HttpMessageEntry;
import solms.xmlvend.messagevalidator.ReportKeyGenerator;
import solms.xmlvend.messagevalidator.ReportSplitter;
import solms.xmlvend.messagevalidator.util.CalendarComparator;
import solms.xmlvend.testreport.Client;
import solms.xmlvend.testreport.ObjectFactory;
import solms.xmlvend.testreport.Period;
import solms.xmlvend.testreport.RawMessage;
import solms.xmlvend.testreport.Response;
import solms.xmlvend.testreport.Server;
import solms.xmlvend.testreport.ServiceRequest;
import solms.xmlvend.testreport.Testreport;

/**
 * Simple report splitter which creates unique reports based on their ClientID, and the server IP
 * address they communicated with. The ClientID is determined using a best-effort mechanism,
 * and it is thus not guaranteed that every message has a determinable ClientID. If not, however,
 * the reports are retained and processed using an 'unknown' clientID. 
 * 
 * NOTE: This class can be refactored to smaller, simpler methods and components. It started as a
 * proof-of-concept, but due to timing constraints is currently being used as-is.
 *
 * @author dawidl
 * @version 0.1 (Jul 1, 2004)
 */
public class ReportSplitterImpl implements ReportSplitter
{

    public static final String UNKNOWN_CLIENT_ID = "unknown";
    public static final String UNKNOWN_SERVICE_NAME = "Unknown_Message_Type";
    protected static String ID = "ReportSplitterImpl";
    protected ReportKeyGenerator keyMaker;
    private CalendarComparator ccomparator;
    
    /** Default Constructor */
    public ReportSplitterImpl()
    {
        keyMaker = new ReportKeyGeneratorImpl();
        ccomparator = new CalendarComparator();
    }

    /* (non-Javadoc)
     * @see solms.xmlvend.messagevalidator.ReportSplitter#produceRawReports(java.util.SortedMap)
     */
    public SortedMap produceRawReports(SortedMap messages) throws Exception
    {
        // Create map of reports
        SortedMap reports = new TreeMap();
        
        // Cycle through messages
        Iterator requests = messages.keySet().iterator();
        while (requests.hasNext())
        {
            try
            {
                // Cast to HTTP Message
                HttpMessageEntry request = (HttpMessageEntry)requests.next();
                
                // Create ServiceRequest
                ObjectFactory factory = new ObjectFactory();
                ServiceRequest serviceRequest = factory.createServiceRequest();
                
                // Copy attributes
                RawMessage reqMsg = factory.createRawMessage();
                reqMsg.setMessage(request.getMessageContent().getValue());
                reqMsg.setTimeStamp(request.getTimestamp());
                serviceRequest.setRawRequest(reqMsg);
                
                // Add response
                HttpMessageEntry response = (HttpMessageEntry)messages.get(request);
                if (response != null)
                {
                    // Copy attributes
                    RawMessage resMsg = factory.createRawMessage();
                    resMsg.setMessage(response.getMessageContent().getValue());
                    resMsg.setTimeStamp(response.getTimestamp());
                    Response resp = factory.createResponse();
                    resp.setRawResponse(resMsg);
                    serviceRequest.setResponse(resp);
                }
                
                // Get Service Name (Name of first message type within SOAP)
                String serviceName = getServiceNameInRawMessage(request.getMessageContent().getValue());
                if (serviceName == null)
                    serviceName = UNKNOWN_SERVICE_NAME;
                serviceRequest.setServiceName(serviceName);
                
                // Get ClientID
                String clientID = getClientIDInRawMessage(request.getMessageContent().getValue());
                if (clientID == null)
                    clientID = UNKNOWN_CLIENT_ID;
                
                // Get server IP Address
                String serverIPAndPort = request.getReceiverHostAndPort();
                
                // Get report key
                String reportKey = keyMaker.generateKey(clientID, serverIPAndPort);
                
                // Get report (or create if it doesn't exist yet)
                Testreport report = (Testreport)reports.get(reportKey);
                if (!reports.containsKey(reportKey))
                {
                    report = factory.createTestreport();
                    
                    // Create client info
                    Client client = factory.createClient();
                    client.setClientId(clientID);
                    client.setIpAddess(request.getSenderHostAndPort());                    
                    report.setClient(client);
                    
                    // Create Server Info
                    Server server = factory.createServer();
                    server.setIpAddess(serverIPAndPort);
                    report.setServer(server);
                    
                    // Set the period values for first message, will be altered later
                    Period period = factory.createPeriod();
                    period.setStart(request.getTimestamp());
                    period.setEnd(response == null ? request.getTimestamp() : response.getTimestamp());
                    report.setPeriod(period);
                    
                    // Add report to map
                    reports.put(reportKey, report);
                }
                
                // Add message to report
                report.getServiceRequest().add(serviceRequest);
                
                // Use comparator to update report dates as necessary
                // REQUEST < Start
                if (ccomparator.compare(report.getPeriod().getStart(), request.getTimestamp()) > 0)
                {
                    report.getPeriod().setStart(request.getTimestamp());
                }
                // REQUEST > End
                if (ccomparator.compare(report.getPeriod().getEnd(), request.getTimestamp()) < 0)
                {
                    report.getPeriod().setEnd(request.getTimestamp());
                }
                // RESPONSE? > End
                if (response != null && ccomparator.compare(report.getPeriod().getEnd(), response.getTimestamp()) < 0)
                {
                    report.getPeriod().setEnd(response.getTimestamp());
                }
                                
            }
            catch (Exception e)
            {
                System.err.println(ID + ": Error processing message: " + e + " (continuing).");
            }
        }
        return reports;
    }

    /**
     * Given a raw (or even faulty) HTTP Message, the method will find the ClientID if it exists,
     * in a very fault-tolerant (non-xml-reliant) way. Because we do not know what type of client
     * ID clients may use, this pattern is based on the EAN-type client id, i.e. assumes a numeric
     * ID of some sort will be used as an attribute on the "clientID" tag.
     * @param value
     * @return
     */
    private String getClientIDInRawMessage(String message)
    {
        if (message == null)
            return null;
        
        // Fairly complex pattern which tries to cater to all possible scenarios, e.g faulty messages,
        // multiple occurrences, illegal spaces, namespace declarations etc.
        
        // Old Pattern
        // Pattern.compile("ClientID[\\sa-zA-Z\"=:]*>[\\s]*([a-zA-Z0-9:\\-_|]*)[\\s]*</.*?ClientID[\\s>]");
        String cidPattern = "<[^>]*clientID[\\s]*[^>]*[\\s]+([-a-zA-Z0-9_]+)=[\"']([0-9]+)[\"'][^>]*>";
        
        Pattern pattern = Pattern.compile( cidPattern );
        Matcher matcher = pattern.matcher(message);
        matcher.find();
        try
        {
            // Found (should return something like "ean.20343234235345"
        		String id = matcher.group(1) + "." + matcher.group(2);
            return id;
        } catch (IllegalStateException e)
        {
            // Not found
        		System.out.println("Client ID unknown: " + e.toString());
            return null;
        }
    }
    
    /**
     * Given a raw (or even faulty) HTTP Message, the method will find the name of the root element
     * of the soap message, within the SOAP envelope and body, e.g. Vend_Request.
     * @param value
     * @return
     */
    private String getServiceNameInRawMessage(String message)
    {
        if (message == null)
            return null;
        
        // Pattern which tries to cater to all possible scenarios, e.g faulty messages,
        // multiple occurrences, illegal spaces, etc. Matches on the first tage after "Body"
        // (which is assumed to be SOAP)
        Pattern pattern = Pattern.compile("[<:]Body[\\s]*[^>]*>[\\s]*<([a-zA-Z0-9:\\-_]+)[^>]*>");
        Matcher matcher = pattern.matcher(message);
        matcher.find();
        try
        {
            // Found
            return matcher.group(1);
        } catch (IllegalStateException e)
        {
            // Not found
            return null;
        }
    } 
}